@@ -59,6 +59,7 @@
 #include <stdint.h>
 #include <cassert>
 #include <utility>
+#include <map>
 
 namespace aria2 {
 
@@ -67,10 +68,15 @@ FtpNegotiationCommand::FtpNegotiationCommand(int32_t cuid,
 					     RequestGroup* requestGroup,
 					     DownloadEngine* e,
 					     const SocketHandle& s,
-					     Seq seq):
+					     Seq seq,
+					     const std::string& baseWorkingDir):
   AbstractCommand(cuid, req, requestGroup, e, s), sequence(seq),
   ftp(new FtpConnection(cuid, socket, req, e->option))
 {
+  ftp->setBaseWorkingDir(baseWorkingDir);
+  if(seq == SEQ_RECV_GREETING) {
+    setTimeout(e->option->getAsInt(PREF_CONNECT_TIMEOUT));
+  }
   disableReadCheckSocket();
   setWriteCheckSocket(socket);
 }
@@ -102,7 +108,7 @@ bool FtpNegotiationCommand::executeInternal() {
     if(e->option->getAsBool(PREF_FTP_PASV)) {
       sequence = SEQ_SEND_PASV;
     } else {
-      sequence = SEQ_SEND_PORT;
+      sequence = SEQ_PREPARE_SERVER_SOCKET;
     }
     return false;
   } else {
@@ -112,7 +118,9 @@ bool FtpNegotiationCommand::executeInternal() {
 }
 
 bool FtpNegotiationCommand::recvGreeting() {
-  socket->setBlockingMode();
+  checkIfConnectionEstablished(socket);
+  setTimeout(e->option->getAsInt(PREF_TIMEOUT));
+  //socket->setBlockingMode();
   disableWriteCheckSocket();
   setReadCheckSocket(socket);
 
@@ -129,8 +137,12 @@ bool FtpNegotiationCommand::recvGreeting() {
 }
 
 bool FtpNegotiationCommand::sendUser() {
-  ftp->sendUser();
-  sequence = SEQ_RECV_USER;
+  if(ftp->sendUser()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_USER;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -152,8 +164,12 @@ bool FtpNegotiationCommand::recvUser() {
 }
 
 bool FtpNegotiationCommand::sendPass() {
-  ftp->sendPass();
-  sequence = SEQ_RECV_PASS;
+  if(ftp->sendPass()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_PASS;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -170,8 +186,12 @@ bool FtpNegotiationCommand::recvPass() {
 }
 
 bool FtpNegotiationCommand::sendType() {
-  ftp->sendType();
-  sequence = SEQ_RECV_TYPE;
+  if(ftp->sendType()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_TYPE;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -183,13 +203,46 @@ bool FtpNegotiationCommand::recvType() {
   if(status != 200) {
     throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
   }
+  sequence = SEQ_SEND_PWD;
+  return true;
+}
+
+bool FtpNegotiationCommand::sendPwd()
+{
+  if(ftp->sendPwd()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_PWD;
+  } else {
+    setWriteCheckSocket(socket);
+  }
+  return false;
+}
+
+bool FtpNegotiationCommand::recvPwd()
+{
+  std::string pwd;
+  unsigned int status = ftp->receivePwdResponse(pwd);
+  if(status == 0) {
+    return false;
+  }
+  if(status != 257) {
+    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+  }
+  ftp->setBaseWorkingDir(pwd);
+  logger->info("CUID#%d - base working directory is '%s'", cuid, pwd.c_str());
   sequence = SEQ_SEND_CWD;
   return true;
 }
 
 bool FtpNegotiationCommand::sendCwd() {
-  ftp->sendCwd();
-  sequence = SEQ_RECV_CWD;
+  // Calling setReadCheckSocket() is needed when the socket is reused, 
+  setReadCheckSocket(socket);
+  if(ftp->sendCwd()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_CWD;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -200,15 +253,63 @@ bool FtpNegotiationCommand::recvCwd() {
   }
   if(status != 250) {
     poolConnection();
+    _requestGroup->increaseAndValidateFileNotFoundCount();
     throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
   }
-  sequence = SEQ_SEND_SIZE;
+  if(e->option->getAsBool(PREF_REMOTE_TIME)) {
+    sequence = SEQ_SEND_MDTM;
+  } else {
+    sequence = SEQ_SEND_SIZE;
+  }
   return true;
 }
 
+bool FtpNegotiationCommand::sendMdtm()
+{
+  if(ftp->sendMdtm()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_MDTM;
+  } else {
+    setWriteCheckSocket(socket);
+  }
+  return false;
+}
+
+bool FtpNegotiationCommand::recvMdtm()
+{
+  Time lastModifiedTime(-1);
+  unsigned int status = ftp->receiveMdtmResponse(lastModifiedTime);
+  if(status == 0) {
+    return false;
+  }
+  if(status == 213) {
+    if(lastModifiedTime.good()) {
+      _requestGroup->updateLastModifiedTime(lastModifiedTime);
+      time_t t = lastModifiedTime.getTime();
+      struct tm* tms = gmtime(&t); // returned struct is statically allocated.
+      if(tms) {
+	logger->debug("MDTM result was parsed as: %s GMT", asctime(tms));
+      } else {
+	logger->debug("gmtime() failed for MDTM result.");
+      }
+    } else {
+      logger->debug("MDTM response was returned, but it seems not to be a time"
+		    " value as in specified in RFC3659.");
+    }
+  } else {
+    logger->info("CUID#%d - MDTM command failed.", cuid);
+  }
+  sequence = SEQ_SEND_SIZE;
+  return true;  
+}
+
 bool FtpNegotiationCommand::sendSize() {
-  ftp->sendSize();
-  sequence = SEQ_RECV_SIZE;
+  if(ftp->sendSize()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_SIZE;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -290,7 +391,7 @@ bool FtpNegotiationCommand::recvSize() {
       if(e->option->getAsBool(PREF_FTP_PASV)) {
 	sequence = SEQ_SEND_PASV;
       } else {
-	sequence = SEQ_SEND_PORT;
+	sequence = SEQ_PREPARE_SERVER_SOCKET;
       }
       return onFileSizeDetermined(0);
 
@@ -301,7 +402,7 @@ bool FtpNegotiationCommand::recvSize() {
   if(e->option->getAsBool(PREF_FTP_PASV)) {
     sequence = SEQ_SEND_PASV;
   } else {
-    sequence = SEQ_SEND_PORT;
+    sequence = SEQ_PREPARE_SERVER_SOCKET;
   }
   return true;
 }
@@ -311,10 +412,21 @@ void FtpNegotiationCommand::afterFileAllocation()
   setReadCheckSocket(socket);
 }
 
+bool FtpNegotiationCommand::prepareServerSocket()
+{
+  serverSocket = ftp->createServerSocket();
+  sequence = SEQ_SEND_PORT;
+  return true;
+}
+
 bool FtpNegotiationCommand::sendPort() {
   afterFileAllocation();
-  serverSocket = ftp->sendPort();
-  sequence = SEQ_RECV_PORT;
+  if(ftp->sendPort(serverSocket)) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_PORT;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -332,8 +444,12 @@ bool FtpNegotiationCommand::recvPort() {
 
 bool FtpNegotiationCommand::sendPasv() {
   afterFileAllocation();
-  ftp->sendPasv();
-  sequence = SEQ_RECV_PASV;
+  if(ftp->sendPasv()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_PASV;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -361,15 +477,19 @@ bool FtpNegotiationCommand::recvPasv() {
 }
 
 bool FtpNegotiationCommand::sendRestPasv(const SegmentHandle& segment) {
-  dataSocket->setBlockingMode();
+  //dataSocket->setBlockingMode();
   setReadCheckSocket(socket);
   disableWriteCheckSocket();
   return sendRest(segment);
 }
 
 bool FtpNegotiationCommand::sendRest(const SegmentHandle& segment) {
-  ftp->sendRest(segment);
-  sequence = SEQ_RECV_REST;
+  if(ftp->sendRest(segment)) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_REST;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -390,8 +510,12 @@ bool FtpNegotiationCommand::recvRest(const SharedHandle<Segment>& segment) {
 }
 
 bool FtpNegotiationCommand::sendRetr() {
-  ftp->sendRetr();
-  sequence = SEQ_RECV_RETR;
+  if(ftp->sendRetr()) {
+    disableWriteCheckSocket();
+    sequence = SEQ_RECV_RETR;
+  } else {
+    setWriteCheckSocket(socket);
+  }
   return false;
 }
 
@@ -401,6 +525,7 @@ bool FtpNegotiationCommand::recvRetr() {
     return false;
   }
   if(status != 150 && status != 125) {
+    _requestGroup->increaseAndValidateFileNotFoundCount();
     throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
   }
   if(e->option->getAsBool(PREF_FTP_PASV)) {
@@ -419,7 +544,7 @@ bool FtpNegotiationCommand::waitConnection()
   disableReadCheckSocket();
   setReadCheckSocket(socket);
   dataSocket.reset(serverSocket->acceptConnection());
-  dataSocket->setBlockingMode();
+  //dataSocket->setBlockingMode();
   sequence = SEQ_NEGOTIATION_COMPLETED;
   return false;
 }
@@ -441,14 +566,24 @@ bool FtpNegotiationCommand::processSequence(const SegmentHandle& segment) {
     return sendType();
   case SEQ_RECV_TYPE:
     return recvType();
+  case SEQ_SEND_PWD:
+    return sendPwd();
+  case SEQ_RECV_PWD:
+    return recvPwd();
   case SEQ_SEND_CWD:
     return sendCwd();
   case SEQ_RECV_CWD:
     return recvCwd();
+  case SEQ_SEND_MDTM:
+    return sendMdtm();
+  case SEQ_RECV_MDTM:
+    return recvMdtm();
   case SEQ_SEND_SIZE:
     return sendSize();
   case SEQ_RECV_SIZE:
     return recvSize();
+  case SEQ_PREPARE_SERVER_SOCKET:
+    return prepareServerSocket();
   case SEQ_SEND_PORT:
     return sendPort();
   case SEQ_RECV_PORT:
@@ -481,7 +616,9 @@ void FtpNegotiationCommand::poolConnection() const
      e->option->getAsBool(PREF_FTP_REUSE_CONNECTION)) {
     std::pair<std::string, uint16_t> peerInfo;
     socket->getPeerInfo(peerInfo);
-    e->poolSocket(peerInfo.first, peerInfo.second, socket);
+    std::map<std::string, std::string> options;
+    options["baseWorkingDir"] = ftp->getBaseWorkingDir();
+    e->poolSocket(peerInfo.first, peerInfo.second, socket, options);
   }
 }
 
