@@ -72,6 +72,8 @@
 #include "FileAllocationIterator.h"
 #include "StringFormat.h"
 #include "A2STR.h"
+#include "URISelector.h"
+#include "InOrderURISelector.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "CheckIntegrityCommand.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -120,6 +122,9 @@ RequestGroup::RequestGroup(const Option* option,
   _haltRequested(false),
   _forceHaltRequested(false),
   _singleHostMultiConnectionEnabled(true),
+  _uriSelector(new InOrderURISelector()),
+  _lastModifiedTime(-1),
+  _fileNotFoundCount(0),
   _option(option),
   _logger(LogFactory::getInstance())
 {
@@ -488,9 +493,9 @@ void RequestGroup::createNextCommandWithAdj(std::deque<Command*>& commands,
     } else {
       numCommand = _numConcurrentCommand;
     }
-    numCommand += numAdj;
     numCommand = std::min(static_cast<int>(_downloadContext->getNumPieces()),
 			  numCommand);
+    numCommand += numAdj;
   }
   if(numCommand > 0) {
     createNextCommand(commands, e, numCommand);
@@ -503,8 +508,8 @@ void RequestGroup::createNextCommand(std::deque<Command*>& commands,
 				     const std::string& method)
 {
   std::deque<std::string> pendingURIs;
-  for(;!_uris.empty() && numCommand--; _uris.pop_front()) {
-    std::string uri = _uris.front();
+  for(; !_uris.empty() && numCommand--; ) {    
+    std::string uri = _uriSelector->select(_uris);
     RequestHandle req(new Request());
     if(req->setUrl(uri)) {
       ServerHostHandle sv;
@@ -967,9 +972,9 @@ void RequestGroup::reportDownloadFinished()
   _logger->notice(MSG_FILE_DOWNLOAD_COMPLETED,
 		  getFilePath().c_str());
 #ifdef ENABLE_BITTORRENT
-  TransferStat stat = calculateStat();
   SharedHandle<BtContext> ctx = dynamic_pointer_cast<BtContext>(_downloadContext);
   if(!ctx.isNull()) {
+    TransferStat stat = calculateStat();
     double shareRatio = ((stat.getAllTimeUploadLength()*10)/getCompletedLength())/10.0;
     _logger->notice(MSG_SHARE_RATIO_REPORT,
 		    shareRatio,
@@ -1017,4 +1022,40 @@ void RequestGroup::removeAcceptType(const std::string& type)
 		     _acceptTypes.end());
 }
 
+void RequestGroup::setURISelector(const SharedHandle<URISelector>& uriSelector)
+{
+  _uriSelector = uriSelector;
+}
+
+void RequestGroup::applyLastModifiedTimeToLocalFiles()
+{
+  if(!_pieceStorage.isNull() && _lastModifiedTime.good()) {
+    time_t t = _lastModifiedTime.getTime();
+    _logger->info("Applying Last-Modified time: %s in local time zone",
+		  ctime(&t));
+    size_t n =
+      _pieceStorage->getDiskAdaptor()->utime(Time(), _lastModifiedTime);
+    _logger->info("Last-Modified attrs of %lu files were updated.",
+		  static_cast<unsigned long>(n));
+  }
+}
+
+void RequestGroup::updateLastModifiedTime(const Time& time)
+{
+  if(time.good() && _lastModifiedTime < time) {
+    _lastModifiedTime = time;
+  }
+}
+
+void RequestGroup::increaseAndValidateFileNotFoundCount()
+{
+  ++_fileNotFoundCount;
+  const unsigned int maxCount = _option->getAsInt(PREF_MAX_FILE_NOT_FOUND);
+  if(maxCount > 0 && _fileNotFoundCount >= maxCount &&
+     _segmentMan->calculateSessionDownloadLength() == 0) {
+    throw DownloadFailureException
+      (StringFormat("Reached max-file-not-found count=%u", maxCount).str());
+  }
+}
+
 } // namespace aria2
