@@ -5,9 +5,9 @@
 #include "Piece.h"
 #include "Range.h"
 #include "Request.h"
-#include "CookieBox.h"
 #include "Option.h"
 #include "array_fun.h"
+#include "CookieStorage.h"
 #include <cppunit/extensions/HelperMacros.h>
 
 namespace aria2 {
@@ -26,11 +26,20 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testUserAgent);
   CPPUNIT_TEST(testAddHeader);
   CPPUNIT_TEST(testAddAcceptType);
+  CPPUNIT_TEST(testEnableAcceptEncoding);
   CPPUNIT_TEST_SUITE_END();
 private:
-
+  SharedHandle<Option> _option;
 public:
-  void setUp() {}
+  void setUp()
+  {
+    _option.reset(new Option());
+    
+    SharedHandle<AuthConfigFactory> authConfigFactory
+      (new AuthConfigFactory(_option.get()));
+    SingletonHolder<SharedHandle<AuthConfigFactory> >::instance
+      (authConfigFactory);
+  }
 
   void testGetStartByte();
   void testGetEndByte();
@@ -43,6 +52,7 @@ public:
   void testUserAgent();
   void testAddHeader();
   void testAddAcceptType();
+  void testEnableAcceptEncoding();
 };
 
 
@@ -54,11 +64,11 @@ void HttpRequestTest::testGetStartByte()
   SharedHandle<Piece> p(new Piece(1, 1024));
   SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
-  CPPUNIT_ASSERT_EQUAL(0LL, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getStartByte());
 
   httpRequest.setSegment(segment);
   
-  CPPUNIT_ASSERT_EQUAL(1024LL, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)1024LL, httpRequest.getStartByte());
 
 }
 
@@ -72,11 +82,11 @@ void HttpRequestTest::testGetEndByte()
   SharedHandle<Piece> piece(new Piece(index, length));
   SharedHandle<Segment> segment(new PiecedSegment(segmentLength, piece));
 
-  CPPUNIT_ASSERT_EQUAL(0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 
   httpRequest.setSegment(segment);
 
-  CPPUNIT_ASSERT_EQUAL(0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 
   SharedHandle<Request> request(new Request());
   request->supportsPersistentConnection(true);
@@ -84,31 +94,24 @@ void HttpRequestTest::testGetEndByte()
 
   httpRequest.setRequest(request);
 
-  CPPUNIT_ASSERT_EQUAL((off_t)segmentLength*index+length-1,
+  CPPUNIT_ASSERT_EQUAL((off_t)(segmentLength*index+length-1),
 		       httpRequest.getEndByte());
 
 
   request->setPipeliningHint(false);
 
-  CPPUNIT_ASSERT_EQUAL(0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 }
 
 void HttpRequestTest::testCreateRequest()
 {
   SharedHandle<Piece> p;
 
-  Option option;
-  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_USER, "aria2user");
-  option.put(PREF_HTTP_PASSWD, "aria2passwd");
-  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
-
-  SharedHandle<AuthConfigFactory> authConfigFactory(new AuthConfigFactory(&option));
-  SingletonHolder<SharedHandle<AuthConfigFactory> >::instance(authConfigFactory);
+  _option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
+  _option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
+  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+  _option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
+  _option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
 
   SharedHandle<Request> request(new Request());
   request->supportsPersistentConnection(true);
@@ -119,6 +122,7 @@ void HttpRequestTest::testCreateRequest()
   SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
 
@@ -218,9 +222,10 @@ void HttpRequestTest::testCreateRequest()
   httpRequest.setSegment(segment);
 
   // enable http auth
-  option.put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
-  
-  httpRequest.configure(&option);
+  _option->put(PREF_HTTP_USER, "aria2user");
+  _option->put(PREF_HTTP_PASSWD, "aria2passwd");
+
+  httpRequest.configure(_option.get());
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -235,9 +240,9 @@ void HttpRequestTest::testCreateRequest()
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // enable http proxy auth
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -251,9 +256,9 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
+  _option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -267,9 +272,9 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
+  _option->put(PREF_HTTP_PROXY_METHOD, V_GET);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -303,9 +308,9 @@ void HttpRequestTest::testCreateRequest()
 
   request->setPipeliningHint(false);
 
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -323,19 +328,13 @@ void HttpRequestTest::testCreateRequest()
 
 void HttpRequestTest::testCreateRequest_ftp()
 {
-  Option option;
-  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_USER, "aria2user");
-  option.put(PREF_HTTP_PASSWD, "aria2passwd");
-  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
-
-  SharedHandle<AuthConfigFactory> authConfigFactory
-    (new AuthConfigFactory(&option));
-  SingletonHolder<SharedHandle<AuthConfigFactory> >::instance(authConfigFactory);
+  _option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
+  _option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
+  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+  _option->put(PREF_FTP_USER, "aria2user");
+  _option->put(PREF_FTP_PASSWD, "aria2passwd");
+  _option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
+  _option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
 
   SharedHandle<Request> request(new Request());
   request->setUrl("ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2");
@@ -344,30 +343,36 @@ void HttpRequestTest::testCreateRequest_ftp()
   SharedHandle<Piece> p(new Piece(0, 1024*1024));
   SharedHandle<Segment> segment
     (new PiecedSegment(1024*1024, p));
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
-  std::string expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  std::string expectedText =
+    "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
+    " HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
+    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // How to enable HTTP proxy authorization in FTP download via HTTP proxy
-  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  _option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
+  _option->put(PREF_HTTP_PROXY_METHOD, V_GET);
+  _option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
 
-  httpRequest.configure(&option);
+  httpRequest.configure(_option.get());
 
-  expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  expectedText =
+    "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
+    " HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
@@ -376,6 +381,7 @@ void HttpRequestTest::testCreateRequest_ftp()
     "Connection: close\r\n"
     "Proxy-Connection: close\r\n"
     "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
+    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
@@ -392,18 +398,21 @@ void HttpRequestTest::testCreateRequest_with_cookie()
 
   Cookie cookie1("name1", "value1", "/archives", "localhost", false);
   Cookie cookie2("name2", "value2", "/archives/download", "localhost", false);
-  Cookie cookie3("name3", "value3", "/archives/download", "tt.localhost", false);
-  Cookie cookie4("name4", "value4", "/archives/download", "tt.localhost", true);
+  Cookie cookie3("name3", "value3", "/archives/download", ".aria2.org", false);
+  Cookie cookie4("name4", "value4", "/archives/", ".aria2.org", true);
 
-  request->cookieBox->add(cookie1);
-  request->cookieBox->add(cookie2);
-  request->cookieBox->add(cookie3);
-  request->cookieBox->add(cookie4);
+  SharedHandle<CookieStorage> st(new CookieStorage());
+  CPPUNIT_ASSERT(st->store(cookie1));
+  CPPUNIT_ASSERT(st->store(cookie2));
+  CPPUNIT_ASSERT(st->store(cookie3));
+  CPPUNIT_ASSERT(st->store(cookie4));
 
   HttpRequest httpRequest;
 
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setCookieStorage(st);
 
   std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -426,35 +435,36 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;\r\n"
+    "Cookie: name2=value2;name1=value1;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setUrl("http://tt.localhost/archives/download/aria2-1.0.0.tar.bz2");
+  request->setUrl("http://www.aria2.org/archives/download/aria2-1.0.0.tar.bz2");
 
   expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
-    "Host: tt.localhost\r\n"
+    "Host: www.aria2.org\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;name3=value3;\r\n"
+    "Cookie: name3=value3;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setUrl("https://tt.localhost/archives/download/aria2-1.0.0.tar.bz2");
+  request->setUrl("https://www.aria2.org/archives/download/"
+		  "aria2-1.0.0.tar.bz2");
 
   expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
-    "Host: tt.localhost\r\n"
+    "Host: www.aria2.org\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;name3=value3;name4=value4;\r\n"
+    "Cookie: name3=value3;name4=value4;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
@@ -466,6 +476,7 @@ void HttpRequestTest::testCreateRequest_query()
   SharedHandle<Request> request(new Request());
   request->setUrl("http://localhost/wiki?id=9ad5109a-b8a5-4edf-9373-56a1c34ae138");
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
 
   std::string expectedText =
@@ -583,12 +594,6 @@ void HttpRequestTest::testIsRangeSatisfied()
 
 void HttpRequestTest::testUserAgent()
 {
-  Option option;
-
-  SharedHandle<AuthConfigFactory> authConfigFactory
-    (new AuthConfigFactory(&option));
-  SingletonHolder<SharedHandle<AuthConfigFactory> >::instance(authConfigFactory);
-
   SharedHandle<Request> request(new Request());
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
@@ -596,6 +601,7 @@ void HttpRequestTest::testUserAgent()
   SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
   httpRequest.setUserAgent("aria2 (Linux)");
@@ -626,6 +632,7 @@ void HttpRequestTest::testAddHeader()
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.addHeader("X-ARIA2: v0.13\nX-ARIA2-DISTRIBUTE: enabled\n");
 
@@ -652,6 +659,7 @@ void HttpRequestTest::testAddAcceptType()
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.addAcceptType(&acceptTypes[0], &acceptTypes[arrayLength(acceptTypes)]);
 
@@ -668,4 +676,34 @@ void HttpRequestTest::testAddAcceptType()
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 }
 
+void HttpRequestTest::testEnableAcceptEncoding()
+{
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+
+  HttpRequest httpRequest;
+  httpRequest.setRequest(request);
+
+  std::string acceptEncodings;
+#ifdef HAVE_LIBZ
+  acceptEncodings += "deflate, gzip";
+#endif // HAVE_LIBZ
+  
+  std::string expectedText =
+    "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n";
+  if(!acceptEncodings.empty()) {
+    expectedText += "Accept-Encoding: "+acceptEncodings+"\r\n";
+  }
+  expectedText +=
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
 } // namespace aria2
