@@ -1,3 +1,7 @@
+#include <iostream>
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "HttpResponse.h"
 #include "prefs.h"
 #include "PiecedSegment.h"
@@ -9,8 +13,7 @@
 #include "A2STR.h"
 #include "Decoder.h"
 #include "DlRetryEx.h"
-#include <iostream>
-#include <cppunit/extensions/HelperMacros.h>
+#include "CookieStorage.h"
 
 namespace aria2 {
 
@@ -40,6 +43,8 @@ class HttpResponseTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testValidateResponse_chunked);
   CPPUNIT_TEST(testHasRetryAfter);
   CPPUNIT_TEST(testProcessRedirect);
+  CPPUNIT_TEST(testRetrieveCookie);
+  CPPUNIT_TEST(testSupportsPersistentConnection);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -69,6 +74,8 @@ public:
   void testValidateResponse_chunked();
   void testHasRetryAfter();
   void testProcessRedirect();
+  void testRetrieveCookie();
+  void testSupportsPersistentConnection();
 };
 
 
@@ -454,4 +461,53 @@ void HttpResponseTest::testProcessRedirect()
   }
 }
 
+void HttpResponseTest::testRetrieveCookie()
+{
+  HttpResponse httpResponse;
+  SharedHandle<HttpHeader> httpHeader(new HttpHeader());
+  httpResponse.setHttpHeader(httpHeader);
+
+  SharedHandle<HttpRequest> httpRequest(new HttpRequest());
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://www.aria2.org/archives/aria2-1.0.0.tar.bz2");
+  httpRequest->setRequest(request);
+  SharedHandle<CookieStorage> st(new CookieStorage());
+  httpRequest->setCookieStorage(st);
+  httpResponse.setHttpRequest(httpRequest);
+
+  httpHeader->put("Set-Cookie", "k1=v1; expires=Sun, 10-Jun-2007 11:00:00 GMT;"
+		  "path=/; domain=.aria2.org;");
+  httpHeader->put("Set-Cookie", "k2=v2; expires=Sun, 01-Jan-09 00:00:00 GMT;"
+		  "path=/; domain=.aria2.org;");
+  httpHeader->put("Set-Cookie", "k3=v3;");
+
+  httpResponse.retrieveCookie();
+
+  CPPUNIT_ASSERT_EQUAL((size_t)2, st->size());
+
+  std::deque<Cookie>::const_iterator citer = st->begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k2=v2"), (*(st->begin())).toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("k3=v3"), (*(st->begin()+1)).toString());
+}
+
+void HttpResponseTest::testSupportsPersistentConnection()
+{
+  HttpResponse httpResponse;
+  SharedHandle<HttpHeader> httpHeader(new HttpHeader());
+  httpResponse.setHttpHeader(httpHeader);
+
+  httpHeader->setVersion("HTTP/1.1");
+  CPPUNIT_ASSERT(httpResponse.supportsPersistentConnection());
+
+  httpHeader->setVersion("HTTP/1.0");
+  CPPUNIT_ASSERT(!httpResponse.supportsPersistentConnection());
+
+  httpHeader->setVersion("HTTP/1.1");
+  httpHeader->put("Connection", "close");
+  CPPUNIT_ASSERT(!httpResponse.supportsPersistentConnection());
+  httpHeader->clearField();
+  httpHeader->put("Connection", "keep-alive");
+  CPPUNIT_ASSERT(httpResponse.supportsPersistentConnection());
+}
+
 } // namespace aria2
